# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class BaseVMGroup(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Machine group base schema.
    """

    class MetaOapg:

        class properties:

            class max_idle_time(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    format = 'time-delta'
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'max_idle_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class auto_terminate_ts(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'auto_terminate_ts':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class id(
                    schemas.UUIDBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'uuid'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'id':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class name(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'name':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class machine_type(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'machine_type':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class disk_size_gb(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'disk_size_gb':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class num_vms(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'num_vms':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class idle_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'idle_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class creation_timestamp(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'creation_timestamp':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class deletion_timestamp(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'deletion_timestamp':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class type(
                    schemas.ComposedSchema,):

                class MetaOapg:

                    @classmethod
                    @functools.lru_cache()
                    def all_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            MachineGroupType,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'type':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class provider_id(
                    schemas.ComposedSchema,):

                class MetaOapg:

                    @classmethod
                    @functools.lru_cache()
                    def all_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            Providers,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'provider_id':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            started = schemas.BoolSchema

            class quota_usage(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    additional_properties = schemas.NumberSchema
                    any_of_0 = schemas.DictSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __getitem__(
                        self, name: typing.Union[
                            str,
                        ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)

                def get_item_oapg(
                        self, name: typing.Union[
                            str,
                        ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[
                        MetaOapg.additional_properties,
                        decimal.Decimal,
                        int,
                        float,
                    ],
                ) -> 'quota_usage':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            __annotations__ = {
                "max_idle_time": max_idle_time,
                "auto_terminate_ts": auto_terminate_ts,
                "id": id,
                "name": name,
                "machine_type": machine_type,
                "disk_size_gb": disk_size_gb,
                "num_vms": num_vms,
                "idle_seconds": idle_seconds,
                "creation_timestamp": creation_timestamp,
                "deletion_timestamp": deletion_timestamp,
                "type": type,
                "provider_id": provider_id,
                "started": started,
                "quota_usage": quota_usage,
            }

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["max_idle_time"]
    ) -> MetaOapg.properties.max_idle_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["auto_terminate_ts"]
    ) -> MetaOapg.properties.auto_terminate_ts:
        ...

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["machine_type"]
    ) -> MetaOapg.properties.machine_type:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["disk_size_gb"]
    ) -> MetaOapg.properties.disk_size_gb:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["num_vms"]
    ) -> MetaOapg.properties.num_vms:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["idle_seconds"]
    ) -> MetaOapg.properties.idle_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["creation_timestamp"]
    ) -> MetaOapg.properties.creation_timestamp:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["deletion_timestamp"]
    ) -> MetaOapg.properties.deletion_timestamp:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["type"]
    ) -> MetaOapg.properties.type:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["provider_id"]
    ) -> MetaOapg.properties.provider_id:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["started"]
    ) -> MetaOapg.properties.started:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["quota_usage"]
    ) -> MetaOapg.properties.quota_usage:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "max_idle_time",
        "auto_terminate_ts",
        "id",
        "name",
        "machine_type",
        "disk_size_gb",
        "num_vms",
        "idle_seconds",
        "creation_timestamp",
        "deletion_timestamp",
        "type",
        "provider_id",
        "started",
        "quota_usage",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["max_idle_time"]
    ) -> typing.Union[MetaOapg.properties.max_idle_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["auto_terminate_ts"]
    ) -> typing.Union[MetaOapg.properties.auto_terminate_ts, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["id"]
    ) -> typing.Union[MetaOapg.properties.id, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["name"]
    ) -> typing.Union[MetaOapg.properties.name, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["machine_type"]
    ) -> typing.Union[MetaOapg.properties.machine_type, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["disk_size_gb"]
    ) -> typing.Union[MetaOapg.properties.disk_size_gb, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["num_vms"]
    ) -> typing.Union[MetaOapg.properties.num_vms, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["idle_seconds"]
    ) -> typing.Union[MetaOapg.properties.idle_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["creation_timestamp"]
    ) -> typing.Union[MetaOapg.properties.creation_timestamp, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["deletion_timestamp"]
    ) -> typing.Union[MetaOapg.properties.deletion_timestamp, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["type"]
    ) -> typing.Union[MetaOapg.properties.type, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["provider_id"]
    ) -> typing.Union[MetaOapg.properties.provider_id, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["started"]
    ) -> typing.Union[MetaOapg.properties.started, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["quota_usage"]
    ) -> typing.Union[MetaOapg.properties.quota_usage, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "max_idle_time",
        "auto_terminate_ts",
        "id",
        "name",
        "machine_type",
        "disk_size_gb",
        "num_vms",
        "idle_seconds",
        "creation_timestamp",
        "deletion_timestamp",
        "type",
        "provider_id",
        "started",
        "quota_usage",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        max_idle_time: typing.Union[MetaOapg.properties.max_idle_time, dict,
                                    frozendict.frozendict, str, date, datetime,
                                    uuid.UUID, int, float, decimal.Decimal,
                                    bool, None, list, tuple, bytes, io.FileIO,
                                    io.BufferedReader,
                                    schemas.Unset] = schemas.unset,
        auto_terminate_ts: typing.Union[MetaOapg.properties.auto_terminate_ts,
                                        dict, frozendict.frozendict, str, date,
                                        datetime, uuid.UUID, int, float,
                                        decimal.Decimal, bool, None, list,
                                        tuple, bytes, io.FileIO,
                                        io.BufferedReader,
                                        schemas.Unset] = schemas.unset,
        id: typing.Union[MetaOapg.properties.id, dict, frozendict.frozendict,
                         str, date, datetime, uuid.UUID, int, float,
                         decimal.Decimal, bool, None, list, tuple, bytes,
                         io.FileIO, io.BufferedReader,
                         schemas.Unset] = schemas.unset,
        name: typing.Union[MetaOapg.properties.name, dict,
                           frozendict.frozendict, str, date, datetime,
                           uuid.UUID, int, float, decimal.Decimal, bool, None,
                           list, tuple, bytes, io.FileIO, io.BufferedReader,
                           schemas.Unset] = schemas.unset,
        machine_type: typing.Union[MetaOapg.properties.machine_type, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        disk_size_gb: typing.Union[MetaOapg.properties.disk_size_gb, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        num_vms: typing.Union[MetaOapg.properties.num_vms, dict,
                              frozendict.frozendict, str, date, datetime,
                              uuid.UUID, int, float, decimal.Decimal, bool,
                              None, list, tuple, bytes, io.FileIO,
                              io.BufferedReader, schemas.Unset] = schemas.unset,
        idle_seconds: typing.Union[MetaOapg.properties.idle_seconds, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        creation_timestamp: typing.Union[MetaOapg.properties.creation_timestamp,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        deletion_timestamp: typing.Union[MetaOapg.properties.deletion_timestamp,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        type: typing.Union[MetaOapg.properties.type, dict,
                           frozendict.frozendict, str, date, datetime,
                           uuid.UUID, int, float, decimal.Decimal, bool, None,
                           list, tuple, bytes, io.FileIO, io.BufferedReader,
                           schemas.Unset] = schemas.unset,
        provider_id: typing.Union[MetaOapg.properties.provider_id, dict,
                                  frozendict.frozendict, str, date, datetime,
                                  uuid.UUID, int, float, decimal.Decimal, bool,
                                  None, list, tuple, bytes, io.FileIO,
                                  io.BufferedReader,
                                  schemas.Unset] = schemas.unset,
        started: typing.Union[MetaOapg.properties.started, bool,
                              schemas.Unset] = schemas.unset,
        quota_usage: typing.Union[MetaOapg.properties.quota_usage, dict,
                                  frozendict.frozendict, str, date, datetime,
                                  uuid.UUID, int, float, decimal.Decimal, bool,
                                  None, list, tuple, bytes, io.FileIO,
                                  io.BufferedReader,
                                  schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'BaseVMGroup':
        return super().__new__(
            cls,
            *_args,
            max_idle_time=max_idle_time,
            auto_terminate_ts=auto_terminate_ts,
            id=id,
            name=name,
            machine_type=machine_type,
            disk_size_gb=disk_size_gb,
            num_vms=num_vms,
            idle_seconds=idle_seconds,
            creation_timestamp=creation_timestamp,
            deletion_timestamp=deletion_timestamp,
            type=type,
            provider_id=provider_id,
            started=started,
            quota_usage=quota_usage,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.machine_group_type import MachineGroupType
from inductiva.client.model.providers import Providers



"""Register CLI commands for autocomplete."""
import argparse
import os

from inductiva import constants
from inductiva._cli import loader, utils


def register(root_parser):
    parser = root_parser.add_parser(
        "autocomplete",
        help="Enables auto-completion for inductiva commands.",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = (
        "Controls the autocomplete behavior for the inductiva package.\n"
        "The `inductiva autocomplete` command allows you to setup "
        "autocompletion for the cli.\n"
        "At the moment we only provide support for zsh.")

    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX)



"""Class to manage the MPI cluster in Google Cloud."""
from absl import logging
from typing import Optional
import datetime

from inductiva.resources import machines_base


class MPICluster(machines_base.BaseMachineGroup):
    """Class to launch and manage an MPI cluster in Google Cloud.

    A MPI cluster is a collection of homogenous machines all working together on
    a common task given the configurations that are launched in Google Cloud.
    Note: The cluster will be available only after calling 'start' method.
    The billing will start only after the machines are started."""

    def __init__(
        self,
        machine_type: str,
        num_machines: int = 2,
        data_disk_gb: int = 10,
        max_idle_time: Optional[datetime.timedelta] = None,
        auto_terminate_ts: Optional[datetime.datetime] = None,
        register: bool = True,
    ) -> None:
        """Create a MPICluster object.

        The register argument is used to indicate if the machine group should
        be registered or if it was already registered. If set as False on
        initialization, then, the machine group is not registered and it
        can not be started in the cloud. This serves has an helper argument for
        retrieving already registered machine groups that can be started, for
        example, when retrieving with the `machines_groups.get` method.
        Users should not set this argument.

        Args:
            machine_type: The type of GC machine to launch. Ex: "e2-standard-4".
              Check https://cloud.google.com/compute/docs/machine-resource for
              information about machine types.
            num_machines: The number of virtual machines to launch.
            data_disk_gb: The size of the disk for user data (in GB).
            max_idle_time: Time without executing any task, after which the
              resource will be terminated.
            auto_terminate_ts: Moment in which the resource will be
              automatically terminated.
        """
        if num_machines < 1:
            raise ValueError(
                "`num_machines` should be a number greater than 0.")

        super().__init__(
            machine_type=machine_type,
            data_disk_gb=data_disk_gb,
            max_idle_time=max_idle_time,
            auto_terminate_ts=auto_terminate_ts,
            register=register,
        )

        # num_machines is the number of machines requested
        self.num_machines = num_machines
        #Number of active machines at the time of
        #the request machine_groups.get()
        self._active_machines = 0
        self.__type = machines_base.ResourceType.MPI.value
        self.__is_elastic = False
        self.__spot = False

        if register:
            logging.info("■ Registering MPICluster configurations:")
            self._register_machine_group(num_vms=self.num_machines,
                                         is_elastic=self.__is_elastic,
                                         spot=self.__spot,
                                         type=self.__type)

    @classmethod
    def from_api_response(cls, resp: dict):
        machine_group = super().from_api_response(resp)
        machine_group.num_machines = int(resp["max_vms"])
        machine_group.__dict__["_active_machines"] = int(resp["num_vms"])
        machine_group.register = False
        return machine_group

    def __repr__(self):
        class_name = self.__class__.__name__
        return f"{class_name}(name=\"{self.name}\")"

    def __str__(self):
        return f"MPI Cluster {self.name} with {self.machine_type} " \
               f"x{self.num_machines} machines"

    def start(self):
        """Start the MPI Cluster."""
        return super().start(
            num_vms=self.num_machines,
            is_elastic=self.__is_elastic,
            spot=self.__spot,
            type=self.__type,
        )

    def terminate(self, verbose: bool = True):
        """Terminates the MPI Cluster."""
        return super().terminate(num_vms=self.num_machines,
                                 is_elastic=self.__is_elastic,
                                 spot=self.__spot,
                                 verbose=verbose)

    def _log_machine_group_info(self):
        super()._log_machine_group_info()
        logging.info("\t· Number of machines:       %s", self.num_machines)
        self.estimate_cloud_cost()

    def estimate_cloud_cost(self):
        """Estimates a cost per hour of the MPI cluster in US dollars.

        This is an estimate of the cost of MPI cluster with the
        specified configurations up in the cloud. The actual cost may vary.

        Returns:
            The estimated cost per hour of the machine group, in US
              dollars ($/h)."""
        #TODO: Contemplate disk size in the price.
        estimated_cost = super()._get_estimated_cost() * self.num_machines
        logging.info("\t· Estimated cloud cost of the MPI cluster: %.3f $/h",
                     estimated_cost)
        return estimated_cost



"""Information about the contents of a Task output archive."""
from dataclasses import dataclass
from typing import List

from inductiva.utils import format_utils


@dataclass
class FileInfo:
    """Information about a file in an output archive."""
    name: str
    size: int
    compressed_size: int


@dataclass
class TaskOutputInfo:
    """Information about the contents of a Task output archive."""
    files: List[FileInfo]
    total_size_bytes: int

    @property
    def n_files(self) -> int:
        return len(self.files)

    @property
    def total_compressed_size_bytes(self) -> int:
        return sum(file.compressed_size for file in self.files)

    def __str__(self) -> str:
        size_header = "Size"
        compressed_header = "Compressed"
        name_header = "Name"

        total_size_formatted = format_utils.bytes_formatter(
            self.total_size_bytes)
        total_compressed_size_formatted = format_utils.bytes_formatter(
            self.total_compressed_size_bytes)

        lines = [
            f"Total size: {total_size_formatted}",
            f"Total compressed size: {total_compressed_size_formatted}",
            f"Number of files: {self.n_files}",
            "Contents:",
            f"  {size_header:<12} {compressed_header:<12} {name_header:<12}",
        ]
        for file in self.files:
            lines.append(
                f"  {format_utils.bytes_formatter(file.size):<12}"
                f" {format_utils.bytes_formatter(file.compressed_size):<12}"
                f" {file.name:<12}")

        return "\n".join(lines)



"""Base class for low-level simulators."""
from typing import Optional
from abc import ABC
import logging

import pathlib

from inductiva import types, tasks, resources
from .methods import list_available_images
from inductiva import commands


def mpi_enabled(cls):
    """Class decorator that adds MPICluster to the supported resources.
    """
    supports = getattr(cls, "_supported_resources", set())
    cls._supported_resources = supports | {resources.MPICluster}  # pylint: disable=protected-access

    return cls


class Simulator(ABC):
    """Base simulator class."""

    _supported_resources = {
        resources.MachineGroup, resources.ElasticMachineGroup
    }
    _logger = logging.getLogger(__name__)

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        if version is not None and not isinstance(version, str):
            raise ValueError("Version must be a string or None.")
        self.api_method_name = ""
        self._version = version
        self._use_dev = bool(use_dev)
        self._image_uri = self._get_image_uri()
        self._logger.info("")

    @property
    def version(self):
        """Get the version of the simulator."""
        return self._version

    @property
    def use_dev(self):
        """Get whether the development version of the simulator is used."""
        return self._use_dev

    @property
    def name(self):
        """Get the name of the simulator."""
        return self.__class__.__name__

    @property
    def image_uri(self):
        """Get the image URI for this simulator."""
        return self._image_uri

    def _get_image_uri(self):
        """Get the appropriate image name for this simulator."""

        img_type = "development" if self._use_dev else "production"
        sim_name = self.name
        name = sim_name.lower()

        available = list_available_images()
        listing = available.get(img_type, {}).get(name, [])

        if self._version is None:
            # kutu does not have a specific tag for the latest version
            # this hack is a workaround to get that version, but it is prone
            # to errors.
            self._version = max(listing)

        if self._version not in listing:
            raise ValueError(
                f"Version {self.version} is not available for simulator {name}."
                f" Available versions are: {listing}.")
        self._logger.info("■ Using %s image of %s version %s", img_type,
                          sim_name, self.version)

        suffix = "_dev" if self._use_dev else ""
        return f"docker://inductiva/kutu:{name}_v{self._version}" + suffix

    @classmethod
    def get_supported_resources(cls):
        """Get the supported computational resources for this simulator."""
        return tuple(cls._supported_resources)

    def override_api_method_prefix(self, prefix: str):
        """Override the API method prefix.

        Example:
            # prefix = "protein_solvation"
            "md.gromacs.run_simulation" becomes
              "protein_solvation.gromacs.run_simulation"

        Args:
            prefix: The new prefix to use.
        """
        last_elements = self.api_method_name.split(".")[1:]
        all_elements = [prefix] + last_elements

        self.api_method_name = ".".join(all_elements)

    def _setup_input_dir(self, input_dir: str):
        """Setup the simulator input directory."""
        input_dir_path = pathlib.Path(input_dir)
        if not input_dir_path.is_dir():
            raise ValueError(
                f"The provided path (\"{input_dir}\") is not a directory.")
        return input_dir_path

    def run(
        self,
        input_dir: str,
        *_args,
        on: Optional[types.ComputationalResources] = None,
        storage_dir: Optional[str] = "",
        resubmit_on_preemption: bool = False,
        extra_metadata: Optional[dict] = None,
        **kwargs,
    ) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory containing the input files.
            _args: Unused in this method, but defined to allow for more
                non-default arguments in method override in subclasses.
            on: The computational resource to launch the simulation in. If None
                the simulation is launched in a machine of the default pool.
            storage_dir: Parent directory for storing simulation
                               results.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
            **kwargs: Additional keyword arguments to be passed to the
                simulation API method.
        """
        input_dir_path = self._setup_input_dir(input_dir)

        self.validate_computational_resources(on)

        if "commands" in kwargs:
            cmds = commands.Command.commands_to_dicts(kwargs["commands"])
            kwargs["commands"] = cmds

        # Get the user-specified image name. If not specified,
        # use the default image name for the current simulator
        container_image = kwargs.pop("container_image", self._image_uri)

        return tasks.run_simulation(
            self.api_method_name,
            input_dir_path,
            simulator=self,
            storage_dir=storage_dir,
            computational_resources=on,
            extra_metadata=extra_metadata,
            container_image=container_image,
            resubmit_on_preemption=resubmit_on_preemption,
            **kwargs,
        )

    def validate_computational_resources(self, resource):
        """Validate the computational resources passed to the run method.

        Args:
            resource: The computational resource to validate.
            valid_resources: The valid computational resources for the simulator
        """

        if resource is not None \
            and not isinstance(resource, tuple(self._supported_resources)):
            raise ValueError(
                "The computational resource is invalid for this simulator. "
                f"Expected one of {self._supported_resources} but got "
                f"{resource}.")



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class Executer(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "vm_type",
            "memory",
            "n_mpi_hosts",
            "cpu_count_logical",
            "cpu_count_physical",
            "uuid",
            "host_type",
            "vm_name",
        }
        
        class properties:
            uuid = schemas.StrSchema
            cpu_count_logical = schemas.IntSchema
            cpu_count_physical = schemas.IntSchema
            memory = schemas.IntSchema
            n_mpi_hosts = schemas.IntSchema
            vm_type = schemas.StrSchema
            vm_name = schemas.StrSchema
            host_type = schemas.StrSchema
            
            
            class error_detail(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'error_detail':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            __annotations__ = {
                "uuid": uuid,
                "cpu_count_logical": cpu_count_logical,
                "cpu_count_physical": cpu_count_physical,
                "memory": memory,
                "n_mpi_hosts": n_mpi_hosts,
                "vm_type": vm_type,
                "vm_name": vm_name,
                "host_type": host_type,
                "error_detail": error_detail,
            }
    
    vm_type: MetaOapg.properties.vm_type
    memory: MetaOapg.properties.memory
    n_mpi_hosts: MetaOapg.properties.n_mpi_hosts
    cpu_count_logical: MetaOapg.properties.cpu_count_logical
    cpu_count_physical: MetaOapg.properties.cpu_count_physical
    uuid: MetaOapg.properties.uuid
    host_type: MetaOapg.properties.host_type
    vm_name: MetaOapg.properties.vm_name
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["uuid"]) -> MetaOapg.properties.uuid: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["cpu_count_logical"]) -> MetaOapg.properties.cpu_count_logical: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["cpu_count_physical"]) -> MetaOapg.properties.cpu_count_physical: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["memory"]) -> MetaOapg.properties.memory: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["n_mpi_hosts"]) -> MetaOapg.properties.n_mpi_hosts: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["vm_type"]) -> MetaOapg.properties.vm_type: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["vm_name"]) -> MetaOapg.properties.vm_name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["host_type"]) -> MetaOapg.properties.host_type: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["error_detail"]) -> MetaOapg.properties.error_detail: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["uuid", "cpu_count_logical", "cpu_count_physical", "memory", "n_mpi_hosts", "vm_type", "vm_name", "host_type", "error_detail", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["uuid"]) -> MetaOapg.properties.uuid: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["cpu_count_logical"]) -> MetaOapg.properties.cpu_count_logical: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["cpu_count_physical"]) -> MetaOapg.properties.cpu_count_physical: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["memory"]) -> MetaOapg.properties.memory: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["n_mpi_hosts"]) -> MetaOapg.properties.n_mpi_hosts: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["vm_type"]) -> MetaOapg.properties.vm_type: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["vm_name"]) -> MetaOapg.properties.vm_name: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["host_type"]) -> MetaOapg.properties.host_type: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["error_detail"]) -> typing.Union[MetaOapg.properties.error_detail, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["uuid", "cpu_count_logical", "cpu_count_physical", "memory", "n_mpi_hosts", "vm_type", "vm_name", "host_type", "error_detail", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        vm_type: typing.Union[MetaOapg.properties.vm_type, str, ],
        memory: typing.Union[MetaOapg.properties.memory, decimal.Decimal, int, ],
        n_mpi_hosts: typing.Union[MetaOapg.properties.n_mpi_hosts, decimal.Decimal, int, ],
        cpu_count_logical: typing.Union[MetaOapg.properties.cpu_count_logical, decimal.Decimal, int, ],
        cpu_count_physical: typing.Union[MetaOapg.properties.cpu_count_physical, decimal.Decimal, int, ],
        uuid: typing.Union[MetaOapg.properties.uuid, str, ],
        host_type: typing.Union[MetaOapg.properties.host_type, str, ],
        vm_name: typing.Union[MetaOapg.properties.vm_name, str, ],
        error_detail: typing.Union[MetaOapg.properties.error_detail, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'Executer':
        return super().__new__(
            cls,
            *_args,
            vm_type=vm_type,
            memory=memory,
            n_mpi_hosts=n_mpi_hosts,
            cpu_count_logical=cpu_count_logical,
            cpu_count_physical=cpu_count_physical,
            uuid=uuid,
            host_type=host_type,
            vm_name=vm_name,
            error_detail=error_detail,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class UserTasksOverview(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "end_date",
            "avg_task_duration",
            "task_success_rate",
            "total_cost",
            "total_compute_time",
            "total_tasks",
            "total_wait_time",
            "preempted_tasks",
            "start_date",
            "username",
        }
        
        class properties:
            start_date = schemas.DateSchema
            end_date = schemas.DateSchema
            username = schemas.StrSchema
            total_tasks = schemas.IntSchema
            total_cost = schemas.NumberSchema
            total_wait_time = schemas.NumberSchema
            total_compute_time = schemas.NumberSchema
            avg_task_duration = schemas.NumberSchema
            task_success_rate = schemas.NumberSchema
            preempted_tasks = schemas.IntSchema
            __annotations__ = {
                "start_date": start_date,
                "end_date": end_date,
                "username": username,
                "total_tasks": total_tasks,
                "total_cost": total_cost,
                "total_wait_time": total_wait_time,
                "total_compute_time": total_compute_time,
                "avg_task_duration": avg_task_duration,
                "task_success_rate": task_success_rate,
                "preempted_tasks": preempted_tasks,
            }
    
    end_date: MetaOapg.properties.end_date
    avg_task_duration: MetaOapg.properties.avg_task_duration
    task_success_rate: MetaOapg.properties.task_success_rate
    total_cost: MetaOapg.properties.total_cost
    total_compute_time: MetaOapg.properties.total_compute_time
    total_tasks: MetaOapg.properties.total_tasks
    total_wait_time: MetaOapg.properties.total_wait_time
    preempted_tasks: MetaOapg.properties.preempted_tasks
    start_date: MetaOapg.properties.start_date
    username: MetaOapg.properties.username
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["start_date"]) -> MetaOapg.properties.start_date: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["end_date"]) -> MetaOapg.properties.end_date: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["username"]) -> MetaOapg.properties.username: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["total_tasks"]) -> MetaOapg.properties.total_tasks: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["total_cost"]) -> MetaOapg.properties.total_cost: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["total_wait_time"]) -> MetaOapg.properties.total_wait_time: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["total_compute_time"]) -> MetaOapg.properties.total_compute_time: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["avg_task_duration"]) -> MetaOapg.properties.avg_task_duration: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["task_success_rate"]) -> MetaOapg.properties.task_success_rate: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["preempted_tasks"]) -> MetaOapg.properties.preempted_tasks: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["start_date", "end_date", "username", "total_tasks", "total_cost", "total_wait_time", "total_compute_time", "avg_task_duration", "task_success_rate", "preempted_tasks", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["start_date"]) -> MetaOapg.properties.start_date: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["end_date"]) -> MetaOapg.properties.end_date: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["username"]) -> MetaOapg.properties.username: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["total_tasks"]) -> MetaOapg.properties.total_tasks: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["total_cost"]) -> MetaOapg.properties.total_cost: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["total_wait_time"]) -> MetaOapg.properties.total_wait_time: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["total_compute_time"]) -> MetaOapg.properties.total_compute_time: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["avg_task_duration"]) -> MetaOapg.properties.avg_task_duration: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["task_success_rate"]) -> MetaOapg.properties.task_success_rate: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["preempted_tasks"]) -> MetaOapg.properties.preempted_tasks: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["start_date", "end_date", "username", "total_tasks", "total_cost", "total_wait_time", "total_compute_time", "avg_task_duration", "task_success_rate", "preempted_tasks", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        end_date: typing.Union[MetaOapg.properties.end_date, str, date, ],
        avg_task_duration: typing.Union[MetaOapg.properties.avg_task_duration, decimal.Decimal, int, float, ],
        task_success_rate: typing.Union[MetaOapg.properties.task_success_rate, decimal.Decimal, int, float, ],
        total_cost: typing.Union[MetaOapg.properties.total_cost, decimal.Decimal, int, float, ],
        total_compute_time: typing.Union[MetaOapg.properties.total_compute_time, decimal.Decimal, int, float, ],
        total_tasks: typing.Union[MetaOapg.properties.total_tasks, decimal.Decimal, int, ],
        total_wait_time: typing.Union[MetaOapg.properties.total_wait_time, decimal.Decimal, int, float, ],
        preempted_tasks: typing.Union[MetaOapg.properties.preempted_tasks, decimal.Decimal, int, ],
        start_date: typing.Union[MetaOapg.properties.start_date, str, date, ],
        username: typing.Union[MetaOapg.properties.username, str, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'UserTasksOverview':
        return super().__new__(
            cls,
            *_args,
            end_date=end_date,
            avg_task_duration=avg_task_duration,
            task_success_rate=task_success_rate,
            total_cost=total_cost,
            total_compute_time=total_compute_time,
            total_tasks=total_tasks,
            total_wait_time=total_wait_time,
            preempted_tasks=preempted_tasks,
            start_date=start_date,
            username=username,
            _configuration=_configuration,
            **kwargs,
        )



"""Constants that can be set for the Inductiva client."""
import os
import pathlib

LOGS_WEBSOCKET_URL = os.environ.get("INDUCTIVA_TASK_LOGS_URL",
                                    "wss://logs.inductiva.ai")

DEFAULT_QUEUE_MACHINE_TYPE = "c2-standard-4"

TASK_KILL_MAX_API_REQUESTS = 5

TASK_KILL_RETRY_SLEEP_SEC = 1.0

MAX_CONFIRMATION_LINES = 5

LOADER_COMMAND_PREFIX = "cmd_"

LOADER_IGNORE_PREFIX = "_"

# when printing the stack trace, how many lines to show
EXCEPTIONS_MAX_TRACEBACK_DEPTH = 2

HOME_DIR = pathlib.Path.home() / ".inductiva"



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class UserMostUsedSimulatorsOverview(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "num_tasks",
            "dedicated_resource_tasks",
            "simulator",
            "shared_resource_tasks",
            "tasks_simulator_cost",
            "success_rate_percentage",
        }

        class properties:
            simulator = schemas.StrSchema
            num_tasks = schemas.IntSchema
            shared_resource_tasks = schemas.IntSchema
            dedicated_resource_tasks = schemas.IntSchema
            tasks_simulator_cost = schemas.NumberSchema
            success_rate_percentage = schemas.NumberSchema
            __annotations__ = {
                "simulator": simulator,
                "num_tasks": num_tasks,
                "shared_resource_tasks": shared_resource_tasks,
                "dedicated_resource_tasks": dedicated_resource_tasks,
                "tasks_simulator_cost": tasks_simulator_cost,
                "success_rate_percentage": success_rate_percentage,
            }

    num_tasks: MetaOapg.properties.num_tasks
    dedicated_resource_tasks: MetaOapg.properties.dedicated_resource_tasks
    simulator: MetaOapg.properties.simulator
    shared_resource_tasks: MetaOapg.properties.shared_resource_tasks
    tasks_simulator_cost: MetaOapg.properties.tasks_simulator_cost
    success_rate_percentage: MetaOapg.properties.success_rate_percentage

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["simulator"]
    ) -> MetaOapg.properties.simulator:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["num_tasks"]
    ) -> MetaOapg.properties.num_tasks:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["shared_resource_tasks"]
    ) -> MetaOapg.properties.shared_resource_tasks:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["dedicated_resource_tasks"]
    ) -> MetaOapg.properties.dedicated_resource_tasks:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["tasks_simulator_cost"]
    ) -> MetaOapg.properties.tasks_simulator_cost:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["success_rate_percentage"]
    ) -> MetaOapg.properties.success_rate_percentage:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "simulator",
        "num_tasks",
        "shared_resource_tasks",
        "dedicated_resource_tasks",
        "tasks_simulator_cost",
        "success_rate_percentage",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["simulator"]
    ) -> MetaOapg.properties.simulator:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["num_tasks"]
    ) -> MetaOapg.properties.num_tasks:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["shared_resource_tasks"]
    ) -> MetaOapg.properties.shared_resource_tasks:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["dedicated_resource_tasks"]
    ) -> MetaOapg.properties.dedicated_resource_tasks:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["tasks_simulator_cost"]
    ) -> MetaOapg.properties.tasks_simulator_cost:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["success_rate_percentage"]
    ) -> MetaOapg.properties.success_rate_percentage:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "simulator",
        "num_tasks",
        "shared_resource_tasks",
        "dedicated_resource_tasks",
        "tasks_simulator_cost",
        "success_rate_percentage",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        num_tasks: typing.Union[
            MetaOapg.properties.num_tasks,
            decimal.Decimal,
            int,
        ],
        dedicated_resource_tasks: typing.Union[
            MetaOapg.properties.dedicated_resource_tasks,
            decimal.Decimal,
            int,
        ],
        simulator: typing.Union[
            MetaOapg.properties.simulator,
            str,
        ],
        shared_resource_tasks: typing.Union[
            MetaOapg.properties.shared_resource_tasks,
            decimal.Decimal,
            int,
        ],
        tasks_simulator_cost: typing.Union[
            MetaOapg.properties.tasks_simulator_cost,
            decimal.Decimal,
            int,
            float,
        ],
        success_rate_percentage: typing.Union[
            MetaOapg.properties.success_rate_percentage,
            decimal.Decimal,
            int,
            float,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'UserMostUsedSimulatorsOverview':
        return super().__new__(
            cls,
            *_args,
            num_tasks=num_tasks,
            dedicated_resource_tasks=dedicated_resource_tasks,
            simulator=simulator,
            shared_resource_tasks=shared_resource_tasks,
            tasks_simulator_cost=tasks_simulator_cost,
            success_rate_percentage=success_rate_percentage,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from inductiva.client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from inductiva.client.model.autoscale_policy import AutoscalePolicy
from inductiva.client.model.backend_version import BackendVersion
from inductiva.client.model.base_vm_group import BaseVMGroup
from inductiva.client.model.campaign import Campaign
from inductiva.client.model.campaign_users_update import CampaignUsersUpdate
from inductiva.client.model.created_user import CreatedUser
from inductiva.client.model.default_machine_group import DefaultMachineGroup
from inductiva.client.model.executer import Executer
from inductiva.client.model.executer_tracker_api_connection_info import ExecuterTrackerAPIConnectionInfo
from inductiva.client.model.executer_tracker_register_info import ExecuterTrackerRegisterInfo
from inductiva.client.model.executer_tracker_token import ExecuterTrackerToken
from inductiva.client.model.file_download_url import FileDownloadUrl
from inductiva.client.model.file_info import FileInfo
from inductiva.client.model.file_upload_url import FileUploadUrl
from inductiva.client.model.http_validation_error import HTTPValidationError
from inductiva.client.model.machine_group_terminate_reason import MachineGroupTerminateReason
from inductiva.client.model.machine_group_terminate_request import MachineGroupTerminateRequest
from inductiva.client.model.machine_group_type import MachineGroupType
from inductiva.client.model.machine_type import MachineType
from inductiva.client.model.order import Order
from inductiva.client.model.output_archive_info import OutputArchiveInfo
from inductiva.client.model.project import Project
from inductiva.client.model.project_create import ProjectCreate
from inductiva.client.model.provider import Provider
from inductiva.client.model.providers import Providers
from inductiva.client.model.quota import Quota
from inductiva.client.model.quota_scope import QuotaScope
from inductiva.client.model.storage_file_info import StorageFileInfo
from inductiva.client.model.storage_sort_by import StorageSortBy
from inductiva.client.model.task import Task
from inductiva.client.model.task_metric_create import TaskMetricCreate
from inductiva.client.model.task_metrics import TaskMetrics
from inductiva.client.model.task_position_in_queue import TaskPositionInQueue
from inductiva.client.model.task_request import TaskRequest
from inductiva.client.model.task_status import TaskStatus
from inductiva.client.model.task_status_code import TaskStatusCode
from inductiva.client.model.task_submitted_info import TaskSubmittedInfo
from inductiva.client.model.tier import Tier
from inductiva.client.model.user import User
from inductiva.client.model.user_activity import UserActivity
from inductiva.client.model.user_api_key import UserApiKey
from inductiva.client.model.user_computation_trend import UserComputationTrend
from inductiva.client.model.user_costs_over_time import UserCostsOverTime
from inductiva.client.model.user_create import UserCreate
from inductiva.client.model.user_most_used_machine_types_overview import UserMostUsedMachineTypesOverview
from inductiva.client.model.user_most_used_simulators_overview import UserMostUsedSimulatorsOverview
from inductiva.client.model.user_task_status_overview import UserTaskStatusOverview
from inductiva.client.model.user_tasks_overview import UserTasksOverview
from inductiva.client.model.vm_group_config import VMGroupConfig
from inductiva.client.model.validation_error import ValidationError
from inductiva.client.model.version_comparison_result import VersionComparisonResult



"""Available machine types and their number of cores."""
from typing import Union
import json

import inductiva
from inductiva.utils import format_utils
from inductiva.client import ApiException
from inductiva.client.apis.tags import compute_api


class ProviderType(format_utils.CaseInsensitiveEnum):
    """Enum to represent the provider of the machine to be launched."""
    GCP = "GCP"
    ICE = "ICE"
    LOCAL = "LOCAL"


def list_available_machines(provider: Union[str, ProviderType]):
    """List all available machines types."""

    resources_available = get_available_machine_types(provider)
    machine_types = []

    for machine in resources_available:
        machine_types.append(machine["machine_type"])

    return tuple(machine_types)


def get_available_machine_types(provider: Union[str, ProviderType],
                                machine_family: str = None):
    """Get all available machine types for a given provider.

    Args:
        provider (str): The provider to list the available machine types.
        machine_family (str): The machine family to filter the specific
            available machine types."""

    provider = ProviderType(provider)

    api_client = compute_api.ComputeApi(inductiva.api.get_client())

    query_params = {"provider_id": provider.value}

    if machine_family is not None:
        query_params["machine_family"] = machine_family

    try:
        resp = api_client.list_available_machine_types(query_params).response

        response_body = json.loads(resp.data.decode("utf-8"))

        return response_body
    except ApiException as e:
        raise e



"""SCHISM module of the API."""
from typing import Optional

from inductiva import types, tasks, simulators


@simulators.simulator.mpi_enabled
class SCHISM(simulators.Simulator):
    """Class to invoke a generic SCHISM simulation on the API."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the SCHISM simulator.
        
        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "schism.schism.run_simulation"

    def run(self,
            input_dir: str,
            num_scribes: int = 1,
            on: Optional[types.ComputationalResources] = None,
            storage_dir: Optional[str] = "",
            use_hwthread: bool = True,
            extra_metadata: Optional[dict] = None,
            n_vcpus: int = None,
            resubmit_on_preemption: bool = False,
            **kwargs) -> tasks.Task:
        """Run the simulation.
        Args:
            input_dir: Path to the directory of the simulation input files.
            num_scribes: The num_scribes as per the simulator documentation.
            # pylint: disable=line-too-long
            https://schism-dev.github.io/schism/master/getting-started/running-model.html
            # pylint: enable=line-too-long
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            storage_dir: Directory for storing simulation results.
            use_hwthread: If specified Open MPI will attempt to discover the
            number of hardware threads on the node, and use that as the
            number of slots available.
            n_vcpus: Number of virtual cpus
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
        """
        return super().run(input_dir,
                           on=on,
                           num_scribes=num_scribes,
                           storage_dir=storage_dir,
                           n_vcpus=n_vcpus,
                           use_hwthread=use_hwthread,
                           extra_metadata=extra_metadata,
                           resubmit_on_preemption=resubmit_on_preemption,
                           **kwargs)



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class TaskMetrics(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:

        class properties:

            class total_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'total_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class container_image_download_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'container_image_download_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class queue_time_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'queue_time_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class computation_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'computation_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_upload_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_upload_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_download_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_download_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_decompression_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_decompression_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class output_compression_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'output_compression_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class output_upload_seconds(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'output_upload_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_zipped_size_bytes(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_zipped_size_bytes':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_size_bytes(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_size_bytes':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class output_total_files(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'output_total_files':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class output_size_bytes(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'output_size_bytes':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class output_zipped_size_bytes(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'output_zipped_size_bytes':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            __annotations__ = {
                "total_seconds":
                    total_seconds,
                "container_image_download_seconds":
                    container_image_download_seconds,
                "queue_time_seconds":
                    queue_time_seconds,
                "computation_seconds":
                    computation_seconds,
                "input_upload_seconds":
                    input_upload_seconds,
                "input_download_seconds":
                    input_download_seconds,
                "input_decompression_seconds":
                    input_decompression_seconds,
                "output_compression_seconds":
                    output_compression_seconds,
                "output_upload_seconds":
                    output_upload_seconds,
                "input_zipped_size_bytes":
                    input_zipped_size_bytes,
                "input_size_bytes":
                    input_size_bytes,
                "output_total_files":
                    output_total_files,
                "output_size_bytes":
                    output_size_bytes,
                "output_zipped_size_bytes":
                    output_zipped_size_bytes,
            }

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["total_seconds"]
    ) -> MetaOapg.properties.total_seconds:
        ...

    @typing.overload
    def __getitem__(
        self,
        name: typing_extensions.Literal["container_image_download_seconds"]
    ) -> MetaOapg.properties.container_image_download_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["queue_time_seconds"]
    ) -> MetaOapg.properties.queue_time_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["computation_seconds"]
    ) -> MetaOapg.properties.computation_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_upload_seconds"]
    ) -> MetaOapg.properties.input_upload_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_download_seconds"]
    ) -> MetaOapg.properties.input_download_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_decompression_seconds"]
    ) -> MetaOapg.properties.input_decompression_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["output_compression_seconds"]
    ) -> MetaOapg.properties.output_compression_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["output_upload_seconds"]
    ) -> MetaOapg.properties.output_upload_seconds:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_zipped_size_bytes"]
    ) -> MetaOapg.properties.input_zipped_size_bytes:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_size_bytes"]
    ) -> MetaOapg.properties.input_size_bytes:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["output_total_files"]
    ) -> MetaOapg.properties.output_total_files:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["output_size_bytes"]
    ) -> MetaOapg.properties.output_size_bytes:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["output_zipped_size_bytes"]
    ) -> MetaOapg.properties.output_zipped_size_bytes:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "total_seconds",
        "container_image_download_seconds",
        "queue_time_seconds",
        "computation_seconds",
        "input_upload_seconds",
        "input_download_seconds",
        "input_decompression_seconds",
        "output_compression_seconds",
        "output_upload_seconds",
        "input_zipped_size_bytes",
        "input_size_bytes",
        "output_total_files",
        "output_size_bytes",
        "output_zipped_size_bytes",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["total_seconds"]
    ) -> typing.Union[MetaOapg.properties.total_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self,
        name: typing_extensions.Literal["container_image_download_seconds"]
    ) -> typing.Union[MetaOapg.properties.container_image_download_seconds,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["queue_time_seconds"]
    ) -> typing.Union[MetaOapg.properties.queue_time_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["computation_seconds"]
    ) -> typing.Union[MetaOapg.properties.computation_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_upload_seconds"]
    ) -> typing.Union[MetaOapg.properties.input_upload_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_download_seconds"]
    ) -> typing.Union[MetaOapg.properties.input_download_seconds,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_decompression_seconds"]
    ) -> typing.Union[MetaOapg.properties.input_decompression_seconds,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["output_compression_seconds"]
    ) -> typing.Union[MetaOapg.properties.output_compression_seconds,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["output_upload_seconds"]
    ) -> typing.Union[MetaOapg.properties.output_upload_seconds, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_zipped_size_bytes"]
    ) -> typing.Union[MetaOapg.properties.input_zipped_size_bytes,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_size_bytes"]
    ) -> typing.Union[MetaOapg.properties.input_size_bytes, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["output_total_files"]
    ) -> typing.Union[MetaOapg.properties.output_total_files, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["output_size_bytes"]
    ) -> typing.Union[MetaOapg.properties.output_size_bytes, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["output_zipped_size_bytes"]
    ) -> typing.Union[MetaOapg.properties.output_zipped_size_bytes,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "total_seconds",
        "container_image_download_seconds",
        "queue_time_seconds",
        "computation_seconds",
        "input_upload_seconds",
        "input_download_seconds",
        "input_decompression_seconds",
        "output_compression_seconds",
        "output_upload_seconds",
        "input_zipped_size_bytes",
        "input_size_bytes",
        "output_total_files",
        "output_size_bytes",
        "output_zipped_size_bytes",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        total_seconds: typing.Union[MetaOapg.properties.total_seconds, dict,
                                    frozendict.frozendict, str, date, datetime,
                                    uuid.UUID, int, float, decimal.Decimal,
                                    bool, None, list, tuple, bytes, io.FileIO,
                                    io.BufferedReader,
                                    schemas.Unset] = schemas.unset,
        container_image_download_seconds: typing.Union[
            MetaOapg.properties.container_image_download_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        queue_time_seconds: typing.Union[MetaOapg.properties.queue_time_seconds,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        computation_seconds: typing.Union[
            MetaOapg.properties.computation_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        input_upload_seconds: typing.Union[
            MetaOapg.properties.input_upload_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        input_download_seconds: typing.Union[
            MetaOapg.properties.input_download_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        input_decompression_seconds: typing.Union[
            MetaOapg.properties.input_decompression_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        output_compression_seconds: typing.Union[
            MetaOapg.properties.output_compression_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        output_upload_seconds: typing.Union[
            MetaOapg.properties.output_upload_seconds, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        input_zipped_size_bytes: typing.Union[
            MetaOapg.properties.input_zipped_size_bytes, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        input_size_bytes: typing.Union[MetaOapg.properties.input_size_bytes,
                                       dict, frozendict.frozendict, str, date,
                                       datetime, uuid.UUID, int, float,
                                       decimal.Decimal, bool, None, list, tuple,
                                       bytes, io.FileIO, io.BufferedReader,
                                       schemas.Unset] = schemas.unset,
        output_total_files: typing.Union[MetaOapg.properties.output_total_files,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        output_size_bytes: typing.Union[MetaOapg.properties.output_size_bytes,
                                        dict, frozendict.frozendict, str, date,
                                        datetime, uuid.UUID, int, float,
                                        decimal.Decimal, bool, None, list,
                                        tuple, bytes, io.FileIO,
                                        io.BufferedReader,
                                        schemas.Unset] = schemas.unset,
        output_zipped_size_bytes: typing.Union[
            MetaOapg.properties.output_zipped_size_bytes, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'TaskMetrics':
        return super().__new__(
            cls,
            *_args,
            total_seconds=total_seconds,
            container_image_download_seconds=container_image_download_seconds,
            queue_time_seconds=queue_time_seconds,
            computation_seconds=computation_seconds,
            input_upload_seconds=input_upload_seconds,
            input_download_seconds=input_download_seconds,
            input_decompression_seconds=input_decompression_seconds,
            output_compression_seconds=output_compression_seconds,
            output_upload_seconds=output_upload_seconds,
            input_zipped_size_bytes=input_zipped_size_bytes,
            input_size_bytes=input_size_bytes,
            output_total_files=output_total_files,
            output_size_bytes=output_size_bytes,
            output_zipped_size_bytes=output_zipped_size_bytes,
            _configuration=_configuration,
            **kwargs,
        )



"""Wrapper class for simulator commands."""
from inductiva import types


class Command:
    """Abstraction class for commands."""

    def __init__(self, cmd: str, *prompts: str):
        """
        Args:
          cmd: A string with the command, e.g, 'gmx pdb2gmx -f protein.pdb'
          *promts: Strings to be used as answers to prompts, e.g, 'y'

        Example:
        >>> cmd = Command("gmx pdb2gmx -f protein.pdb", "y", "y")
        """
        if not isinstance(cmd, str):
            raise ValueError("cmd argument must be a string.")

        if not all(isinstance(prompt, str) for prompt in prompts):
            raise ValueError("Prompts must be all strings.")

        self.cmd = cmd
        self.prompts = list(prompts)

    def to_dict(self):
        """
        Convert this command to a dictionary representation.
        
        Returns:
        A dictionary with {"cmd": self.cmd, "prompts": self.prompts}

        Example:
        >>> cmd = Command("gmx pdb2gmx -f protein.pdb", "y", "y")
        >>> cmd.to_dict()
        >>> {"cmd": "gmx pdb2gmx -f protein.pdb", prompts: ["y", "y"]}
        """
        return self.__dict__.copy()

    @staticmethod
    def commands_to_dicts(commands: types.Commands):
        """
        Convert the given commands to a list of dictionaries.

        Example:
        >>> commands = [Command("run", "y"), "terminate"]
        >>> simulator.commands_to_dict(commands)
        >>> [{"cmd": "run", "prompts": ["y"]},
        ...  {"cmd": "terminate", "prompts": []}]
        """
        return [
            cmd.to_dict()
            if isinstance(cmd, Command) else Command(cmd).to_dict()
            for cmd in commands
        ]



import typing_extensions

from inductiva.client.paths import PathValues
from inductiva.client.apis.paths.tasks_auth import TasksAuth
from inductiva.client.apis.paths.tasks_submit import TasksSubmit
from inductiva.client.apis.paths.tasks_task_id_input_upload_url import TasksTaskIdInputUploadUrl
from inductiva.client.apis.paths.tasks_task_id_input_uploaded import TasksTaskIdInputUploaded
from inductiva.client.apis.paths.tasks_task_id_input import TasksTaskIdInput
from inductiva.client.apis.paths.tasks_task_id import TasksTaskId
from inductiva.client.apis.paths.tasks import Tasks
from inductiva.client.apis.paths.tasks_task_id_status import TasksTaskIdStatus
from inductiva.client.apis.paths.tasks_task_id_position_in_queue import TasksTaskIdPositionInQueue
from inductiva.client.apis.paths.tasks_task_id_output_list import TasksTaskIdOutputList
from inductiva.client.apis.paths.tasks_task_id_download_output_url import TasksTaskIdDownloadOutputUrl
from inductiva.client.apis.paths.tasks_task_id_output import TasksTaskIdOutput
from inductiva.client.apis.paths.tasks_task_id_resubmit import TasksTaskIdResubmit
from inductiva.client.apis.paths.tasks_task_id_kill import TasksTaskIdKill
from inductiva.client.apis.paths.tasks_task_id_disable_logs import TasksTaskIdDisableLogs
from inductiva.client.apis.paths.admin_users import AdminUsers
from inductiva.client.apis.paths.admin_users_email_api_key import AdminUsersEmailApiKey
from inductiva.client.apis.paths.admin_users_email import AdminUsersEmail
from inductiva.client.apis.paths.admin_users_email_campaign_campaign import AdminUsersEmailCampaignCampaign
from inductiva.client.apis.paths.admin_users_campaign import AdminUsersCampaign
from inductiva.client.apis.paths.admin_users_username_storage_size_fs import AdminUsersUsernameStorageSizeFs
from inductiva.client.apis.paths.admin_users_username_storage_size import AdminUsersUsernameStorageSize
from inductiva.client.apis.paths.admin_users_username_tasks import AdminUsersUsernameTasks
from inductiva.client.apis.paths.admin_groups import AdminGroups
from inductiva.client.apis.paths.admin_groups_active import AdminGroupsActive
from inductiva.client.apis.paths.admin_groups_default import AdminGroupsDefault
from inductiva.client.apis.paths.admin_groups_default_machine_group_id import AdminGroupsDefaultMachineGroupId
from inductiva.client.apis.paths.admin_providers import AdminProviders
from inductiva.client.apis.paths.admin_providers_provider_id import AdminProvidersProviderId
from inductiva.client.apis.paths.admin_active_tasks import AdminActiveTasks
from inductiva.client.apis.paths.admin_executer_tracker_token import AdminExecuterTrackerToken
from inductiva.client.apis.paths.admin_groups_machine_group_id_terminate import AdminGroupsMachineGroupIdTerminate
from inductiva.client.apis.paths.executer_tracker_register import ExecuterTrackerRegister
from inductiva.client.apis.paths.executer_tracker_machine_id import ExecuterTrackerMachineId
from inductiva.client.apis.paths.executer_tracker_machine_id_task import ExecuterTrackerMachineIdTask
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_ack import ExecuterTrackerMachineIdTaskTaskIdAck
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_message import ExecuterTrackerMachineIdTaskTaskIdMessage
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_message_unblock import ExecuterTrackerMachineIdTaskTaskIdMessageUnblock
from inductiva.client.apis.paths.executer_tracker_machine_id_event import ExecuterTrackerMachineIdEvent
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_download_input_url import ExecuterTrackerMachineIdTaskTaskIdDownloadInputUrl
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_upload_output_url import ExecuterTrackerMachineIdTaskTaskIdUploadOutputUrl
from inductiva.client.apis.paths.executer_tracker_machine_id_task_task_id_metric import ExecuterTrackerMachineIdTaskTaskIdMetric
from inductiva.client.apis.paths.compute_group import ComputeGroup
from inductiva.client.apis.paths.compute_type import ComputeType
from inductiva.client.apis.paths.compute_group_start import ComputeGroupStart
from inductiva.client.apis.paths.compute_price import ComputePrice
from inductiva.client.apis.paths.compute_groups import ComputeGroups
from inductiva.client.apis.paths.compute_group_status import ComputeGroupStatus
from inductiva.client.apis.paths.compute_machine_types import ComputeMachineTypes
from inductiva.client.apis.paths.compute_group_name import ComputeGroupName
from inductiva.client.apis.paths.storage_size import StorageSize
from inductiva.client.apis.paths.storage_contents import StorageContents
from inductiva.client.apis.paths.version import Version
from inductiva.client.apis.paths.version_check import VersionCheck
from inductiva.client.apis.paths.users_quotas import UsersQuotas
from inductiva.client.apis.paths.users_info import UsersInfo
from inductiva.client.apis.paths.projects import Projects
from inductiva.client.apis.paths.projects_name import ProjectsName
from inductiva.client.apis.paths.metrics_users_username_activity import MetricsUsersUsernameActivity
from inductiva.client.apis.paths.metrics_users_username_cost_over_time import MetricsUsersUsernameCostOverTime
from inductiva.client.apis.paths.metrics_users_username_task_status_overview import MetricsUsersUsernameTaskStatusOverview
from inductiva.client.apis.paths.metrics_users_username_computation_time_trend import MetricsUsersUsernameComputationTimeTrend
from inductiva.client.apis.paths.metrics_users_username_tasks_overview import MetricsUsersUsernameTasksOverview
from inductiva.client.apis.paths.metrics_users_username_most_used_machine_types import MetricsUsersUsernameMostUsedMachineTypes
from inductiva.client.apis.paths.metrics_users_username_most_used_simulators_overview import MetricsUsersUsernameMostUsedSimulatorsOverview

PathToApi = typing_extensions.TypedDict(
    'PathToApi', {
        PathValues.TASKS_AUTH:
            TasksAuth,
        PathValues.TASKS_SUBMIT:
            TasksSubmit,
        PathValues.TASKS_TASK_ID_INPUT_UPLOAD_URL:
            TasksTaskIdInputUploadUrl,
        PathValues.TASKS_TASK_ID_INPUT_UPLOADED:
            TasksTaskIdInputUploaded,
        PathValues.TASKS_TASK_ID_INPUT:
            TasksTaskIdInput,
        PathValues.TASKS_TASK_ID:
            TasksTaskId,
        PathValues.TASKS:
            Tasks,
        PathValues.TASKS_TASK_ID_STATUS:
            TasksTaskIdStatus,
        PathValues.TASKS_TASK_ID_POSITION_IN_QUEUE:
            TasksTaskIdPositionInQueue,
        PathValues.TASKS_TASK_ID_OUTPUT_LIST:
            TasksTaskIdOutputList,
        PathValues.TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL:
            TasksTaskIdDownloadOutputUrl,
        PathValues.TASKS_TASK_ID_OUTPUT:
            TasksTaskIdOutput,
        PathValues.TASKS_TASK_ID_RESUBMIT:
            TasksTaskIdResubmit,
        PathValues.TASKS_TASK_ID_KILL:
            TasksTaskIdKill,
        PathValues.TASKS_TASK_ID_DISABLE_LOGS:
            TasksTaskIdDisableLogs,
        PathValues.ADMIN_USERS:
            AdminUsers,
        PathValues.ADMIN_USERS_EMAIL_API_KEY:
            AdminUsersEmailApiKey,
        PathValues.ADMIN_USERS_EMAIL:
            AdminUsersEmail,
        PathValues.ADMIN_USERS_EMAIL_CAMPAIGN_CAMPAIGN:
            AdminUsersEmailCampaignCampaign,
        PathValues.ADMIN_USERS_CAMPAIGN:
            AdminUsersCampaign,
        PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE_FS:
            AdminUsersUsernameStorageSizeFs,
        PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE:
            AdminUsersUsernameStorageSize,
        PathValues.ADMIN_USERS_USERNAME_TASKS:
            AdminUsersUsernameTasks,
        PathValues.ADMIN_GROUPS:
            AdminGroups,
        PathValues.ADMIN_GROUPS_ACTIVE:
            AdminGroupsActive,
        PathValues.ADMIN_GROUPS_DEFAULT:
            AdminGroupsDefault,
        PathValues.ADMIN_GROUPS_DEFAULT_MACHINE_GROUP_ID:
            AdminGroupsDefaultMachineGroupId,
        PathValues.ADMIN_PROVIDERS:
            AdminProviders,
        PathValues.ADMIN_PROVIDERS_PROVIDER_ID:
            AdminProvidersProviderId,
        PathValues.ADMIN_ACTIVE_TASKS:
            AdminActiveTasks,
        PathValues.ADMIN_EXECUTERTRACKER_TOKEN:
            AdminExecuterTrackerToken,
        PathValues.ADMIN_GROUPS_MACHINE_GROUP_ID_TERMINATE:
            AdminGroupsMachineGroupIdTerminate,
        PathValues.EXECUTERTRACKER_REGISTER:
            ExecuterTrackerRegister,
        PathValues.EXECUTERTRACKER_MACHINE_ID:
            ExecuterTrackerMachineId,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK:
            ExecuterTrackerMachineIdTask,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_ACK:
            ExecuterTrackerMachineIdTaskTaskIdAck,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE:
            ExecuterTrackerMachineIdTaskTaskIdMessage,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK:
            ExecuterTrackerMachineIdTaskTaskIdMessageUnblock,
        PathValues.EXECUTERTRACKER_MACHINE_ID_EVENT:
            ExecuterTrackerMachineIdEvent,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL:
            ExecuterTrackerMachineIdTaskTaskIdDownloadInputUrl,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL:
            ExecuterTrackerMachineIdTaskTaskIdUploadOutputUrl,
        PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_METRIC:
            ExecuterTrackerMachineIdTaskTaskIdMetric,
        PathValues.COMPUTE_GROUP:
            ComputeGroup,
        PathValues.COMPUTE_TYPE:
            ComputeType,
        PathValues.COMPUTE_GROUP_START:
            ComputeGroupStart,
        PathValues.COMPUTE_PRICE:
            ComputePrice,
        PathValues.COMPUTE_GROUPS:
            ComputeGroups,
        PathValues.COMPUTE_GROUP_STATUS:
            ComputeGroupStatus,
        PathValues.COMPUTE_MACHINE_TYPES:
            ComputeMachineTypes,
        PathValues.COMPUTE_GROUP_NAME:
            ComputeGroupName,
        PathValues.STORAGE_SIZE:
            StorageSize,
        PathValues.STORAGE_CONTENTS:
            StorageContents,
        PathValues.VERSION:
            Version,
        PathValues.VERSIONCHECK:
            VersionCheck,
        PathValues.USERS_QUOTAS:
            UsersQuotas,
        PathValues.USERS_INFO:
            UsersInfo,
        PathValues.PROJECTS:
            Projects,
        PathValues.PROJECTS_NAME:
            ProjectsName,
        PathValues.METRICS_USERS_USERNAME_ACTIVITY:
            MetricsUsersUsernameActivity,
        PathValues.METRICS_USERS_USERNAME_COST_OVER_TIME:
            MetricsUsersUsernameCostOverTime,
        PathValues.METRICS_USERS_USERNAME_TASK_STATUS_OVERVIEW:
            MetricsUsersUsernameTaskStatusOverview,
        PathValues.METRICS_USERS_USERNAME_COMPUTATION_TIME_TREND:
            MetricsUsersUsernameComputationTimeTrend,
        PathValues.METRICS_USERS_USERNAME_TASKS_OVERVIEW:
            MetricsUsersUsernameTasksOverview,
        PathValues.METRICS_USERS_USERNAME_MOST_USED_MACHINE_TYPES:
            MetricsUsersUsernameMostUsedMachineTypes,
        PathValues.METRICS_USERS_USERNAME_MOST_USED_SIMULATORS_OVERVIEW:
            MetricsUsersUsernameMostUsedSimulatorsOverview,
    })

path_to_api = PathToApi({
    PathValues.TASKS_AUTH:
        TasksAuth,
    PathValues.TASKS_SUBMIT:
        TasksSubmit,
    PathValues.TASKS_TASK_ID_INPUT_UPLOAD_URL:
        TasksTaskIdInputUploadUrl,
    PathValues.TASKS_TASK_ID_INPUT_UPLOADED:
        TasksTaskIdInputUploaded,
    PathValues.TASKS_TASK_ID_INPUT:
        TasksTaskIdInput,
    PathValues.TASKS_TASK_ID:
        TasksTaskId,
    PathValues.TASKS:
        Tasks,
    PathValues.TASKS_TASK_ID_STATUS:
        TasksTaskIdStatus,
    PathValues.TASKS_TASK_ID_POSITION_IN_QUEUE:
        TasksTaskIdPositionInQueue,
    PathValues.TASKS_TASK_ID_OUTPUT_LIST:
        TasksTaskIdOutputList,
    PathValues.TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL:
        TasksTaskIdDownloadOutputUrl,
    PathValues.TASKS_TASK_ID_OUTPUT:
        TasksTaskIdOutput,
    PathValues.TASKS_TASK_ID_RESUBMIT:
        TasksTaskIdResubmit,
    PathValues.TASKS_TASK_ID_KILL:
        TasksTaskIdKill,
    PathValues.TASKS_TASK_ID_DISABLE_LOGS:
        TasksTaskIdDisableLogs,
    PathValues.ADMIN_USERS:
        AdminUsers,
    PathValues.ADMIN_USERS_EMAIL_API_KEY:
        AdminUsersEmailApiKey,
    PathValues.ADMIN_USERS_EMAIL:
        AdminUsersEmail,
    PathValues.ADMIN_USERS_EMAIL_CAMPAIGN_CAMPAIGN:
        AdminUsersEmailCampaignCampaign,
    PathValues.ADMIN_USERS_CAMPAIGN:
        AdminUsersCampaign,
    PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE_FS:
        AdminUsersUsernameStorageSizeFs,
    PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE:
        AdminUsersUsernameStorageSize,
    PathValues.ADMIN_USERS_USERNAME_TASKS:
        AdminUsersUsernameTasks,
    PathValues.ADMIN_GROUPS:
        AdminGroups,
    PathValues.ADMIN_GROUPS_ACTIVE:
        AdminGroupsActive,
    PathValues.ADMIN_GROUPS_DEFAULT:
        AdminGroupsDefault,
    PathValues.ADMIN_GROUPS_DEFAULT_MACHINE_GROUP_ID:
        AdminGroupsDefaultMachineGroupId,
    PathValues.ADMIN_PROVIDERS:
        AdminProviders,
    PathValues.ADMIN_PROVIDERS_PROVIDER_ID:
        AdminProvidersProviderId,
    PathValues.ADMIN_ACTIVE_TASKS:
        AdminActiveTasks,
    PathValues.ADMIN_EXECUTERTRACKER_TOKEN:
        AdminExecuterTrackerToken,
    PathValues.ADMIN_GROUPS_MACHINE_GROUP_ID_TERMINATE:
        AdminGroupsMachineGroupIdTerminate,
    PathValues.EXECUTERTRACKER_REGISTER:
        ExecuterTrackerRegister,
    PathValues.EXECUTERTRACKER_MACHINE_ID:
        ExecuterTrackerMachineId,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK:
        ExecuterTrackerMachineIdTask,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_ACK:
        ExecuterTrackerMachineIdTaskTaskIdAck,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE:
        ExecuterTrackerMachineIdTaskTaskIdMessage,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK:
        ExecuterTrackerMachineIdTaskTaskIdMessageUnblock,
    PathValues.EXECUTERTRACKER_MACHINE_ID_EVENT:
        ExecuterTrackerMachineIdEvent,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL:
        ExecuterTrackerMachineIdTaskTaskIdDownloadInputUrl,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL:
        ExecuterTrackerMachineIdTaskTaskIdUploadOutputUrl,
    PathValues.EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_METRIC:
        ExecuterTrackerMachineIdTaskTaskIdMetric,
    PathValues.COMPUTE_GROUP:
        ComputeGroup,
    PathValues.COMPUTE_TYPE:
        ComputeType,
    PathValues.COMPUTE_GROUP_START:
        ComputeGroupStart,
    PathValues.COMPUTE_PRICE:
        ComputePrice,
    PathValues.COMPUTE_GROUPS:
        ComputeGroups,
    PathValues.COMPUTE_GROUP_STATUS:
        ComputeGroupStatus,
    PathValues.COMPUTE_MACHINE_TYPES:
        ComputeMachineTypes,
    PathValues.COMPUTE_GROUP_NAME:
        ComputeGroupName,
    PathValues.STORAGE_SIZE:
        StorageSize,
    PathValues.STORAGE_CONTENTS:
        StorageContents,
    PathValues.VERSION:
        Version,
    PathValues.VERSIONCHECK:
        VersionCheck,
    PathValues.USERS_QUOTAS:
        UsersQuotas,
    PathValues.USERS_INFO:
        UsersInfo,
    PathValues.PROJECTS:
        Projects,
    PathValues.PROJECTS_NAME:
        ProjectsName,
    PathValues.METRICS_USERS_USERNAME_ACTIVITY:
        MetricsUsersUsernameActivity,
    PathValues.METRICS_USERS_USERNAME_COST_OVER_TIME:
        MetricsUsersUsernameCostOverTime,
    PathValues.METRICS_USERS_USERNAME_TASK_STATUS_OVERVIEW:
        MetricsUsersUsernameTaskStatusOverview,
    PathValues.METRICS_USERS_USERNAME_COMPUTATION_TIME_TREND:
        MetricsUsersUsernameComputationTimeTrend,
    PathValues.METRICS_USERS_USERNAME_TASKS_OVERVIEW:
        MetricsUsersUsernameTasksOverview,
    PathValues.METRICS_USERS_USERNAME_MOST_USED_MACHINE_TYPES:
        MetricsUsersUsernameMostUsedMachineTypes,
    PathValues.METRICS_USERS_USERNAME_MOST_USED_SIMULATORS_OVERVIEW:
        MetricsUsersUsernameMostUsedSimulatorsOverview,
})



# coding: utf-8
"""


    Generated by: https://openapi-generator.tech
"""

from dataclasses import dataclass
import typing_extensions
import urllib3
from urllib3._collections import HTTPHeaderDict

from inductiva.client import api_client, exceptions
from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401

from inductiva.client.model.http_validation_error import HTTPValidationError

from . import path

# Path params
TaskIdSchema = schemas.StrSchema
RequestRequiredPathParams = typing_extensions.TypedDict(
    'RequestRequiredPathParams', {
        'task_id': typing.Union[
            TaskIdSchema,
            str,
        ],
    })
RequestOptionalPathParams = typing_extensions.TypedDict(
    'RequestOptionalPathParams', {}, total=False)


class RequestPathParams(RequestRequiredPathParams, RequestOptionalPathParams):
    pass


request_path_task_id = api_client.PathParameter(
    name="task_id",
    style=api_client.ParameterStyle.SIMPLE,
    schema=TaskIdSchema,
    required=True,
)
_auth = [
    'APIKeyHeader',
]
SchemaFor200ResponseBodyApplicationJson = schemas.AnyTypeSchema


@dataclass
class ApiResponseFor200(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: typing.Union[
        SchemaFor200ResponseBodyApplicationJson,
    ]
    headers: schemas.Unset = schemas.unset


_response_for_200 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor200,
    content={
        'application/json':
            api_client.MediaType(schema=SchemaFor200ResponseBodyApplicationJson
                                ),
    },
)
SchemaFor422ResponseBodyApplicationJson = HTTPValidationError


@dataclass
class ApiResponseFor422(api_client.ApiResponse):
    response: urllib3.HTTPResponse
    body: typing.Union[
        SchemaFor422ResponseBodyApplicationJson,
    ]
    headers: schemas.Unset = schemas.unset


_response_for_422 = api_client.OpenApiResponse(
    response_cls=ApiResponseFor422,
    content={
        'application/json':
            api_client.MediaType(schema=SchemaFor422ResponseBodyApplicationJson
                                ),
    },
)
_status_code_to_response = {
    '200': _response_for_200,
    '422': _response_for_422,
}
_all_accept_content_types = ('application/json',)


class BaseApi(api_client.Api):

    @typing.overload
    def _delete_task_oapg(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
            ApiResponseFor200,
    ]:
        ...

    @typing.overload
    def _delete_task_oapg(
        self,
        skip_deserialization: typing_extensions.Literal[True],
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization:
        ...

    @typing.overload
    def _delete_task_oapg(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
            ApiResponseFor200,
            api_client.ApiResponseWithoutDeserialization,
    ]:
        ...

    def _delete_task_oapg(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        """
        Delete Task
        :param skip_deserialization: If true then api_response.response will be set but
            api_response.body and api_response.headers will not be deserialized into schema
            class instances
        """
        self._verify_typed_dict_inputs_oapg(RequestPathParams, path_params)
        used_path = path.value

        _path_params = {}
        for parameter in (request_path_task_id,):
            parameter_data = path_params.get(parameter.name, schemas.unset)
            if parameter_data is schemas.unset:
                continue
            serialized_data = parameter.serialize(parameter_data)
            _path_params.update(serialized_data)

        for k, v in _path_params.items():
            used_path = used_path.replace('{%s}' % k, v)

        _headers = HTTPHeaderDict()
        # TODO add cookie handling
        if accept_content_types:
            for accept_content_type in accept_content_types:
                _headers.add('Accept', accept_content_type)

        response = self.api_client.call_api(
            resource_path=used_path,
            method='delete'.upper(),
            headers=_headers,
            auth_settings=_auth,
            stream=stream,
            timeout=timeout,
        )

        if skip_deserialization:
            api_response = api_client.ApiResponseWithoutDeserialization(
                response=response)
        else:
            response_for_status = _status_code_to_response.get(
                str(response.status))
            if response_for_status:
                api_response = response_for_status.deserialize(
                    response, self.api_client.configuration)
            else:
                api_response = api_client.ApiResponseWithoutDeserialization(
                    response=response)

        if not 200 <= response.status <= 299:
            raise exceptions.ApiException(status=response.status,
                                          reason=response.reason,
                                          api_response=api_response)

        return api_response


class DeleteTask(BaseApi):
    # this class is used by api classes that refer to endpoints with operationId fn names

    @typing.overload
    def delete_task(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
            ApiResponseFor200,
    ]:
        ...

    @typing.overload
    def delete_task(
        self,
        skip_deserialization: typing_extensions.Literal[True],
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization:
        ...

    @typing.overload
    def delete_task(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
            ApiResponseFor200,
            api_client.ApiResponseWithoutDeserialization,
    ]:
        ...

    def delete_task(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        return self._delete_task_oapg(path_params=path_params,
                                      accept_content_types=accept_content_types,
                                      stream=stream,
                                      timeout=timeout,
                                      skip_deserialization=skip_deserialization)


class ApiFordelete(BaseApi):
    # this class is used by api classes that refer to endpoints by path and http method names

    @typing.overload
    def delete(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: typing_extensions.Literal[False] = ...,
    ) -> typing.Union[
            ApiResponseFor200,
    ]:
        ...

    @typing.overload
    def delete(
        self,
        skip_deserialization: typing_extensions.Literal[True],
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
    ) -> api_client.ApiResponseWithoutDeserialization:
        ...

    @typing.overload
    def delete(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = ...,
    ) -> typing.Union[
            ApiResponseFor200,
            api_client.ApiResponseWithoutDeserialization,
    ]:
        ...

    def delete(
        self,
        path_params: RequestPathParams = frozendict.frozendict(),
        accept_content_types: typing.Tuple[str] = _all_accept_content_types,
        stream: bool = False,
        timeout: typing.Optional[typing.Union[int, typing.Tuple]] = None,
        skip_deserialization: bool = False,
    ):
        return self._delete_task_oapg(path_params=path_params,
                                      accept_content_types=accept_content_types,
                                      stream=stream,
                                      timeout=timeout,
                                      skip_deserialization=skip_deserialization)
